import { IObservable, SectionNub, Session, BiefRegime } from "jalhyd";

import { FormDefSection } from "../form-def-section";
import { FieldSet } from "../../fieldset";
import { FormulaireBase } from "./form-base";

export class FormulaireBief extends FormulaireBase {

    private _formSection: FormDefSection;

    /** id du select configurant le régime */
    private _regimeSelectId: string;

    constructor() {
        super();
        this._formSection = new FormDefSection(this);
        // default properties
        this._props["regime"] = BiefRegime.Fluvial;
    }

    protected parseOptions(json: {}) {
        super.parseOptions(json);
        this._formSection.parseOptions(json);

        // id du select configurant la méthode de résolution
        this._regimeSelectId = this.getOption(json, "regimeSelectId");
    }

    public afterParseFieldset(fs: FieldSet) {
        this._formSection.afterParseFieldset(fs);

        // si le FieldSet contient le select de méthode de résolution
        if (this._regimeSelectId) {
            const sel = fs.getFormulaireNodeById(this._regimeSelectId);
            if (sel) {
                // on abonne le formulaire aux propriétés du FieldSet
                fs.properties.addObserver(this);
            }
        }
    }

    // interface Observer

    update(sender: IObservable, data: any) {

        super.update(sender, data);

        if (sender instanceof FieldSet && data.action === "propertyChange") {
            switch (sender.id) {
                case "fs_section":
                    // replace underlying section without replacing whole Nub
                    const newSect = Session.getInstance().createSection(data.value);
                    (this._currentNub as SectionNub).setSection(newSect);
                    // reflect changes in GUI
                    for (const fs of this.allFieldsets) {
                        // show / hide dependent fields
                        fs.updateFields();
                    }
                    this.reset();
                    break;

                case "fs_water_line":
                    console.log("reset water line");
                    this.reset();
                    // Either Z1 or Z2 is calculable, depending on Regime
                    this.getFieldsetById("fs_condlim").updateFields();
                    break;
            }
        }
    }
}
